# $Id: setup_rgd.pl,v 1.2 2008/01/18 10:56:20 mbinwade Exp $
# $Copyright: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

# The RGd configurator
# ====================
#
#  RGd is a provider that is installed in an agent. This agent will 
# automatically talk to the local 5xx installation when the RGd provider 
# gets loaded.
#
# RGd has the following capabalities
# - It can silently log on to the 3.2 installation.
# - It publishes a CORBA object for the 3.2 installation. 
# - It registers the 3.2 installation as an agent called 'localhost' with 
#   Genesis. CLIs can query for an agnet named 'localhost' on a particular 
#   host in the domain to talk the 3.2 installation.
# - It can fetch objects from the 3.2 installation and return them as 3.3
#   objects
# - It can perform operation on vxsvc service
# - It can do event registration and filtering on behalf of the 3.2 installation
#
# Pre-requisites:
# - VEA 3.2 installation
# - VRTSobc33 package.
# - At least one agent should be installed and it should have joined the domain.
#
# Usage:
# perl rgd_config.pl -a,--agent <agent> {-i,--install | -u,--un-install }
#
# The script return with exit code 0 if it was successful.
# If the script 'dies' it returns an exit code of 255.

use Getopt::Long;
use strict;

# Literals
my $VXSVC = "/opt/VRTSob/bin/vxsvc";
my $VXPAL = "/opt/VRTSobc/pal33/bin/vxpal";
my $VXREGCTL = "/opt/VRTSobc/pal33/bin/vxregctl";

my $install = 0;
my $un_install = 0;
my $agent = "";
my %opts = ('agent' => \$agent, 'install' => \$install, 'un-install' => \$un_install);

GetOptions(\%opts, qw(agent=s install un-install));
if ($agent eq "")
{
    usage();
}


my $AGENT_ROOT = "Software/Veritas/VRTSobc/pal33/Agents";

if ( ($install eq 0 && $un_install eq 0) ||
     ($install eq 1 && $un_install eq 1))
{
    usage();
}

if ( $install eq 1)
{
    install_rgd();
}
elsif ($un_install eq 1)
{
    common_checks();
    un_install_rgd();
    exit(0);
}
else
{
    usage();
}

sub common_checks()
{
    # Check if VRTSobc33 is installed
    -f $VXPAL or die "ERROR: VRTSobc33 package is not installed on host\n";
    
    # Check if VRTSobc 3.3.648.0 and above is installed.
    open(VERSION, "$VXPAL -v 2>&1 |");
    my $version = <VERSION>;
    $version =~ /^3.3/ or die "ERROR: VEA 3.3 not installed on host\n";

    # RGd feature works only after 3.3.648.0
    my @version = split /\./, $version;
    $version[2] >= 648 or die "ERROR: VRTSobc package version should be 3.3.648.0 or above\n";

    # Check if the agent specified is installed
    my $command = ("$VXREGCTL openkey $AGENT_ROOT/$agent 2>/dev/null");
    system($command);
    $? eq 0 or die "ERROR: Agent $agent not installed on host\n";
}

sub install_checks()
{
    # Check if VRTSob is installed
    -f $VXSVC or die "ERROR: VRTSob package is not installed on host\n";

    # Check if VEA 3.2 is installed
    open(VERSION, "$VXSVC -v|");
    if (<VERSION> !~ /^3.2|^3.0/)
    {
        my $command = "$VXREGCTL openkey $AGENT_ROOT/$agent/Providers/rgd 2>/dev/null";
        system($command);
        if (!$?)
        {
            # Uninstall RGD if VRTSob version is nolonger 3.2/3.0 and RGD entry is present.
            # This handles the case where SF4.x with LMH is upgraded to SF5.x
            un_install_rgd();
         }

        die "ERROR: VEA 3.2 or 3.0 not installed on host\n";
    }
    common_checks();

    # Check if the agent specified has LeafProvider installed
    my $command = "$VXREGCTL openkey $AGENT_ROOT/$agent/Providers/LeafRegistration 2>/dev/null";
    system($command);
    $? eq 0 or die "ERROR: Agent $agent does not have leaf provider loaded\n";
}

sub usage()
{
    print "RGd is a provider that should be installed in an existing agent.\n";
    print "Usage: perl rgd_config.pl -a,--agent <agent> { -i,--install | -u,--un-install }\n";
    print "   eg: perl rgd_config.pl --agent grid --install\n";
    exit(255);
}

# Un-installation
# ===============
#
# 1. Check if the agent specified is installed.
# 2. Remove the RGD entry from the agent installation.
sub un_install_rgd
{
    # Remove RGD value to the agent.
    my $command = "$VXREGCTL deletevalue $AGENT_ROOT/$agent RGD 2>/dev/null";
    system($command);

    #Remove IMAGEFILE entry
    my $command = "$VXREGCTL deletevalue $AGENT_ROOT/$agent/Providers/rgd IMAGEFILE 2>/dev/null";
    system($command);
    
    #Remove CSM_ENGAGE entry
    my $command = "$VXREGCTL deletevalue $AGENT_ROOT/$agent CSM_ENGAGE 2>/dev/null";
    system($command);
    
    # Remove rgd provider entries.
    my $command = "$VXREGCTL deletekey $AGENT_ROOT/$agent/Providers/rgd 2>/dev/null";
    system($command);

    print "\nSUCCESS: Agent $agent has been successfully un-configured as an RGd.\n";
}

# Installation 
# ============
#
# 1. Check if there is a 3.2 VEA installation on the host.
# 2. Check if VRTSobc33 package (> 646) is installed.
# 3. Check if the agent specified is installed.
# 4. Check if it has the LeafProvider installed.
# 5. Add rgd provider entry in the agent providers
# 6. Add RGD flag to the agent registry.
#
sub install_rgd
{

    install_checks();

    # Add RGD value to the agent.
    my $command = "$VXREGCTL setvalue $AGENT_ROOT/$agent RGD REG_INT 1 2>/dev/null";
    system($command);
    $? eq 0 or die "ERROR: Updation of registry for rgd failed\n";

    # Add rgd provider entries.
    my $command = "$VXREGCTL createkey $AGENT_ROOT/$agent/Providers/rgd 2>/dev/null";
    system($command);
    $? eq 0 or die "ERROR: Updation of registry for rgd failed\n";

    # Add the IMAGEFILE entry
    my $os_name = `uname`;
    my $LIB_NAME;
    if ($os_name =~ /HP-UX/)
    {
        $LIB_NAME = "librgd.sl";
    }
    else
    {
        $LIB_NAME = "librgd.so";
    }

    my $command = "$VXREGCTL setvalue $AGENT_ROOT/$agent/Providers/rgd IMAGEFILE REG_SZ /opt/VRTSobc/pal33/lib/$LIB_NAME 2>/dev/null";
    system($command);
    $? eq 0 or die "ERROR: Updation of registry for rgd failed\n";

    # TODO: Ideally this entry should be setup in Gridnode in CSM env.
    my $command = "$VXREGCTL setvalue $AGENT_ROOT/$agent CSM_ENGAGE REG_SZ \"\" 2>/dev/null";
    system($command);
    $? eq 0 or die "ERROR: Updation of registry for rgd failed\n";

    print "\nSUCCESS: Agent $agent has been successfully configured as an RGd.\n";
    exit(0);
}
